using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace DestructionGolf
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class DestructionGolfGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // Game World

        /// <summary>
        /// Font for drawing messages etc
        /// </summary>
        SpriteFont font;

        /// <summary>
        /// Contains the scrolling background
        /// </summary>
        Texture2D backTexture;
        /// <summary>
        /// Contains the draw size of the background. 
        /// Will be wider than the screen so we can pan it.
        /// </summary>
        Rectangle backRectangle;
        /// <summary>
        /// Maximum scroll speed when the user is panning left and right
        /// </summary>
        float backScrollSpeed;

        /// <summary>
        /// Texture of the single target
        /// </summary>
        Texture2D targetTexture;
        /// <summary>
        /// Used to draw the target and detect collisions
        /// </summary>
        Rectangle targetRectangle;

        /// <summary>
        /// Texture for the ball.
        /// </summary>
        Texture2D ballTexture;
        /// <summary>
        /// Used to draw the ball and detect collisions.
        /// </summary>
        Rectangle ballRectangle;

        /// <summary>
        /// Current ball position.
        /// Converted to integer and used to position 
        /// the ballRectangle for drawing
        /// </summary>
        float ballX, ballY;

        /// <summary>
        /// Current ball speed.
        /// Updates the ball position when it is flying
        /// </summary>
        float ballXSpeed, ballYSpeed;

        /// <summary>
        /// Maximum ball speed.
        /// Used when launching the ball. 
        /// </summary>
        float ballXMaxSpeed, ballYMaxSpeed;

        /// <summary>
        /// Ball acceleration.
        /// Updates the ball speed when it is flying.
        /// Changing ballYAccel will affect the strength of gravity
        /// Changing ballXaccel will affect the wind 
        /// </summary>

        float ballXAccel, ballYAccel;

        /// <summary>
        /// True when the ball is stopped and we
        /// are aiming the shot
        /// </summary>
        bool aimingShot;

        /// <summary>
        /// Used to transform the SpriteBatch draw of the 
        /// playfield. By putting a translation in this 
        /// matrix we can pan the whole screen left and right
        /// </summary>
        Matrix transformation;

        /// <summary>
        /// Amount of the translation being applied to the 
        /// transformation. When aiming this is controlled
        /// by the right thumstick. When the ball is flying
        /// this is controlled by the ball position so that
        /// the pan follows the ball.
        /// </summary>
        float xOffset = 0;

        // display dimensions
        /// <summary>
        /// Size of the screen. Used to scale the game objects
        /// and detect edges.
        /// </summary>
        int width;
        int height;

        /// <summary>
        /// Size of the margin between the ball and the edge of the
        /// screen when panning.
        /// </summary>
        int scrollMargin;


        // Sound Effects

        /// <summary>
        /// Played when the ball hits the target
        /// </summary>
        SoundEffect bangSound;

        public DestructionGolfGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Load the display font
            font = Content.Load<SpriteFont>("MessageFont");

            // Set up the sizes
            width = GraphicsDevice.Viewport.Width;
            height = GraphicsDevice.Viewport.Height;
            scrollMargin = width / 10;

            // Set the pan value to 0
            transformation = Matrix.CreateTranslation(0, 0, 0);


            backTexture = Content.Load<Texture2D>(@"images\Background");
            backRectangle = new Rectangle(0, 0, width * 15, height);
            backScrollSpeed = (float)width / 400;

            targetTexture = Content.Load<Texture2D>(@"images\Target");
            targetRectangle = new Rectangle(550, 300, width / 6, width / 6);

            ballTexture = Content.Load<Texture2D>(@"images\Ball");
            ballRectangle = new Rectangle(0, 0, width / 20, width / 20);
            aimingShot = true;
            ballXMaxSpeed = (float)width / 70;
            ballYMaxSpeed = (float)width / 70;
            ballXAccel = 0;
            ballYAccel = (float)width / 10000;

            bangSound = Content.Load<SoundEffect>(@"sounds\explodeSound");

            resetBall();
        }

        /// <summary>
        /// Puts the ball back in the starting position
        /// </summary>
        private void resetBall()
        {
            xOffset = 0;
            ballX = 100;
            ballY = 500;
            aimingShot = true;
        }


        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            GamePadState gamePad = GamePad.GetState(PlayerIndex.One);

            if (aimingShot)
            {
                // Aiming the shot
                // Pan the display left and right using the right thumbstick
                // This changes the xOffset value that is used to translate
                // the screen
                xOffset = xOffset - gamePad.ThumbSticks.Right.X * backScrollSpeed;

                // Press A to fire the ball
                if (gamePad.Buttons.A == ButtonState.Pressed)
                {
                    // Set the initial speed of the ball from the left thumbstick
                    // position
                    ballXSpeed = gamePad.ThumbSticks.Left.X * ballXMaxSpeed;
                    ballYSpeed = -gamePad.ThumbSticks.Left.Y * ballYMaxSpeed;
                    aimingShot = false;
                }
            }
            else
            {
                // Shot is in the air

                // Update the X and Y position of the ball based on its
                // present speed
                ballX += ballXSpeed;
                ballY += ballYSpeed;

                // Apply acceleration to the ball speed
                ballXSpeed += ballXAccel;
                ballYSpeed += ballYAccel;

                // Check if the ball has reached the right hand edge
                if (ballRectangle.Right + scrollMargin > width)
                {
                    // need to scroll the screen
                    xOffset = width - (ballRectangle.Right + scrollMargin) ;
                }

                // If we have hit a target play the sound and reset the ball
                if (ballRectangle.Intersects(targetRectangle))
                {
                    bangSound.Play();
                    resetBall();
                }

                // If the ball has gone off the bottom of the screen
                // reset it
                if (ballY > height)
                {
                    resetBall();
                }
            }

            // Position the draw rectangle to the nearest integer
            ballRectangle.X = (int)(ballX + 0.5f);
            ballRectangle.Y = (int)(ballY + 0.5f);

            // Make a translation of the screen to perform the pan
            transformation = Matrix.CreateTranslation(xOffset, 0, 0);

            base.Update(gameTime);
        }

        private void drawString(string message)
        {
            int xPos = (int) ((width - font.MeasureString(message).X) / 2);
            spriteBatch.DrawString(font, message, new Vector2(xPos, 0), Color.White);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // Begin drawing with a transformed SpriteBatch

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.None, transformation);

            spriteBatch.Draw(backTexture, Vector2.Zero, Color.White);

            spriteBatch.Draw(targetTexture, targetRectangle, Color.White);

            spriteBatch.Draw(ballTexture, ballRectangle, Color.White);

            spriteBatch.End();

            // This draw will not be scrolled, so all this content
            // will always be displayed

            spriteBatch.Begin();

            drawString("Destruction Golf");

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
